@@ -8,6 +8,7 @@
 #include "plv8.h"
 #include "plv8_param.h"
 #include <sstream>
+#include <string>
 
 extern "C" {
 #define delete		delete_
@@ -112,9 +113,6 @@ SetCallback(Handle<ObjectTemplate> obj, const char *name,
 
 class SubTranBlock
 {
-private:
-	ResourceOwner		m_resowner;
-	MemoryContext		m_mcontext;
 public:
 	SubTranBlock();
 	void enter();
@@ -159,8 +157,6 @@ SPIResultToValue(int status)
 }
 
 SubTranBlock::SubTranBlock()
-	: m_resowner(NULL),
-	  m_mcontext(NULL)
 {}
 
 void
@@ -169,11 +165,7 @@ SubTranBlock::enter()
 	if (!IsTransactionOrTransactionBlock())
 		throw js_error("out of transaction");
 
-	m_resowner = CurrentResourceOwner;
-	m_mcontext = CurrentMemoryContext;
 	BeginInternalSubTransaction(NULL);
-	/* Do not want to leave the previous memory context */
-	MemoryContextSwitchTo(m_mcontext);
 
 }
 
@@ -185,14 +177,7 @@ SubTranBlock::exit(bool success)
 	else
 		RollbackAndReleaseCurrentSubTransaction();
 
-	MemoryContextSwitchTo(m_mcontext);
-	CurrentResourceOwner = m_resowner;
 
-	/*
-	 * AtEOSubXact_SPI() should not have popped any SPI context, but just
-	 * in case it did, make sure we remain connected.
-	 */
-	SPI_restore_connection();
 }
 
 JSONObject::JSONObject()
@@ -315,16 +300,19 @@ plv8_Elog(const Arguments& args)
 		return ThrowError("invalid error level");
 	}
 
-	std::ostringstream	stream;
-
+	std::string msg;
+	std::string buf;
 	for (int i = 1; i < args.Length(); i++)
 	{
-		if (i > 1)
-			stream << ' ';
-		stream << CString(args[i]);
+		if (i > 1){
+			msg += " ";
+		}
+		if (!CString::toStdString(args[i],buf)){
+			return Undefined();
+		}
+		msg += buf;
 	}
-
-	const char	   *message = stream.str().c_str();
+	const char	*message = msg.c_str();
 
 	if (elevel != ERROR)
 	{
@@ -434,12 +422,12 @@ plv8_execute_params(const char *sql, Handle<Array> params)
 }
 
 static Handle<Array>
-convertArgsToArray(const Arguments &args, int start)
+convertArgsToArray(const Arguments &args, int start, int downshift)
 {
 	Local<Array> result = Array::New(args.Length() - start);
 	for (int i = start; i < args.Length(); i++)
 	{
-		result->Set(i, args[i]);
+		result->Set(i - downshift, args[i]);
 	}
 	return result;
 }
@@ -463,7 +451,7 @@ plv8_Execute(const Arguments &args)
 		if (args[1]->IsArray())
 			params = Handle<Array>::Cast(args[1]);
 		else /* Consume trailing elements as an array. */
-			params = convertArgsToArray(args, 1);
+			params = convertArgsToArray(args, 1, 1);
 	}
 
 	int				nparam = params.IsEmpty() ? 0 : params->Length();
@@ -481,6 +469,7 @@ plv8_Execute(const Arguments &args)
 	PG_CATCH();
 	{
 		subtran.exit(false);
+		SPI_pop_conditional(true);
 		throw pg_error();
 	}
 	PG_END_TRY();
@@ -508,7 +497,7 @@ plv8_Prepare(const Arguments &args)
 		if (args[1]->IsArray())
 			array = Handle<Array>::Cast(args[1]);
 		else /* Consume trailing elements as an array. */
-			array = convertArgsToArray(args, 1);
+			array = convertArgsToArray(args, 1, 0);
 		arraylen = array->Length();
 		types = (Oid *) palloc(sizeof(Oid) * arraylen);
 	}
@@ -518,7 +507,11 @@ plv8_Prepare(const Arguments &args)
 		CString			typestr(array->Get(i));
 		int32			typemod;
 
+#if PG_VERSION_NUM >= 90400
+		parseTypeString(typestr, &types[i], &typemod, false);
+#else
 		parseTypeString(typestr, &types[i], &typemod);
+#endif
 	}
 
 	PG_TRY();
@@ -587,7 +580,7 @@ plv8_PlanCursor(const Arguments &args)
 		if (args[0]->IsArray())
 			params = Handle<Array>::Cast(args[0]);
 		else
-			params = convertArgsToArray(args, 0);
+			params = convertArgsToArray(args, 0, 0);
 		nparam = params->Length();
 	}
 
@@ -699,7 +692,7 @@ plv8_PlanExecute(const Arguments &args)
 		if (args[0]->IsArray())
 			params = Handle<Array>::Cast(args[0]);
 		else
-			params = convertArgsToArray(args, 0);
+			params = convertArgsToArray(args, 0, 0);
 		nparam = params->Length();
 	}
 
@@ -926,13 +919,14 @@ static Handle<v8::Value>
 plv8_ReturnNext(const Arguments& args)
 {
 	Handle<v8::Object>	self = args.This();
-	Converter *conv = static_cast<Converter *>(
-			Handle<External>::Cast(
-				self->GetInternalField(PLV8_INTNL_CONV))->Value());
+	Handle<v8::Value>	conv_value = self->GetInternalField(PLV8_INTNL_CONV);
 
-	if (conv == NULL)
+	if (!conv_value->IsExternal())
 		throw js_error("return_next called in context that cannot accept a set");
 
+	Converter *conv = static_cast<Converter *>(
+			Handle<External>::Cast(conv_value)->Value());
+
 	Tuplestorestate *tupstore = static_cast<Tuplestorestate *>(
 			Handle<External>::Cast(
 				self->GetInternalField(PLV8_INTNL_TUPSTORE))->Value());
